#!/usr/bin/env python
# pbmpcd.py
# (c) Dub Spencer - http://arton.cunst.net/mail.html?pbmpcd
# mpd / bemused bridge
#
# 
#
# licensed to you under terms of the
# GNU GENERAL PUBLIC LICENSE, Version 2, June 1991.

from bluetooth import *
from struct import *
import mpdclient2


## a helper function

def send_dir(d = ''):
	"""Send a directorie's entries to the bemused window.
	
	While iterating dont care for tree structure:
	- every directory is the first child,
	- no siblings, last childs etc.
	"""
	
	# send root directory
	if d == '':
		ntype = 0 # root
		ftype = 1 # expanded directory
		code = (ntype<<4) | ftype
		f = '>7sBx'
		r = pack(f, 'LISTACK', code)
		n = client_sock.send(r)
		if calcsize(f) > n:
			print 'Short LISTACK'
	# send a child directory
	else:
		ntype = 0 # root too, child?
		ftype = 1 # expanded directory
		code = (ntype<<4) | ftype
		f = '>7sB%dsx' % len(d)
		r = pack(f, 'LISTACK', code, d)
		n = client_sock.send(r)
		if calcsize(f) > n:
			print 'Short LISTACK '

	# send directory entries
	l = m.lsinfo(d)
	for t in l:
		ntype = 1 # first child
		if t.type == 'directory':
			ftype = 3 # unexpanded directory
			path = t.directory
		elif t.type == 'playlist':
			ftype = 2 # a file
			path = t.playlist
		elif t.type == 'file':
			ftype = 2 # a file
			path = t.file
		else:
			print 'New Type: ' + t
		code = (ntype<<4) | ftype
		# strip basename
		start = path.rfind('/') + 1
		stop = len(path)
		name = path[start:stop]
		f = '>B%dsx' % len(name)
		r = pack(f, code, name)
		n = client_sock.send(r)
		if calcsize(f) > n:
			print 'Short DIRENTRY ', name
	# send end of list
	ntype = 255
	f = '>Bx'
	r = pack(f, ntype)
	n = client_sock.send(r)
	if calcsize(f) > n:
		print 'Short LISTEND'


## MAIN

try:
	m = mpdclient2.connect()
except:
	print 'Could not connect to MPD, shutting down.'
	sys.exit()

server_sock=BluetoothSocket(RFCOMM)
server_sock.bind(('', PORT_ANY))
server_sock.listen(1)
port = server_sock.getsockname()[1]

uuid = '71b5077b-d636-4b47-a0ca-d31e8330db65'
advertise_service(server_sock, 'MPD ',
	service_id = uuid,
	service_classes = [ uuid, SERIAL_PORT_CLASS ],
	profiles = [ SERIAL_PORT_PROFILE ])

print 'Waiting for connection on RFCOMM channel %d' % port
client_sock, client_info = server_sock.accept()
print 'Accepted connection from ', client_info

while True:
	try:
		data = client_sock.recv(1024)
	except BluetoothError:
		client_sock.close()
		print 'Waiting for connection on RFCOMM channel %d' % port
		client_sock, client_info = server_sock.accept()
		print 'Accepted connection from ', client_info
		continue
	try:
		m.ping()
	except EOFError:
		print 'MPD connection lost, reconnecting.'
		try:
			m = mpdclient2.connect()
		except:
			print 'Could not connect.'
			break;

	# Bemused Commands
	cmd = data[0:4]
	# send cursory song info
	if cmd == 'INF2':
		s = m.status()
		if s.state == 'stop':
			st = 0 # state stop
			tc = 0 # song time
			ts = 0 # song length
		elif s.state == 'pause':
			st = 0 # state pause
			tc, ts = s.time.split(':', 2)
		else: # play
			st = 1 # state play
			tc, ts = s.time.split(':', 2)
		t = m.currentsong()
		if len(t) == 0:
			title = '###\n#####'
		else:
			title = t.artist + '\n' + t.title
		f = '>7sBIIBB%dsx' % len(title)
		r = pack (f, 'INFOACK', st, int(ts), int(tc),
			int(s.random), int(s.repeat), title)
		n = client_sock.send(r)
		if calcsize(f) > n:
			print 'Short INF2ACK'
	# send playback volume
	elif cmd == 'GVOL':
		s = m.status()
		f = '>7sB'
		v = int(s.volume) * 255 / 100
		r = pack(f, 'GVOLACK', v)
		n = client_sock.send(r)
		if calcsize(f) > n:
			print 'Short GVOLACK'
	# send current playlist
	elif cmd == 'PLST':
		s = m.status()
		if s.state == 'stop':
			p = 0
		else:
			p = int(s.song)
		l = m.playlistinfo()
		a = '#\n' # playlist header
		for t in l:
			a = a + t.title + '\n'
		f = '>7sH%dsx' % len(a)
		r = pack(f, 'PLSTACK', p, a)
		n = client_sock.send(r)
		if calcsize(f) > n:
			print 'Short PLSTACK'
	# start playback
	elif cmd == 'STRT':
		m.play()
	# stop playback
	elif cmd == 'STOP':
		m.stop()
	# pause playback
	elif cmd == 'PAUS':
		s = m.status()
		if s.state == 'pause':
			m.pause(False)
		else:
			m.pause(True)
	# play next track
	elif cmd == 'NEXT':
		m.next()
	# play previous track
	elif cmd == 'PREV':
		m.previous()
	# wind 10 secs
	elif cmd == 'FFWD':
		s = m.status()
		if s.state == 'stop':
			continue
		tc, ts = s.time.split(':', 2)
		tw = int(tc) + 10
		m.seekid(int(s.songid), tw)
	# rewind 10 secs
	elif cmd == 'RWND':
		s = m.status()
		if s.state == 'stop':
			continue
		tc, ts = s.time.split(':', 2)
		tw = int(tc) - 10
		m.seekid(int(s.songid), tw)
	# set playback volume
	elif cmd == 'VOLM':
		f = '>4sB'
		k, v = unpack(f, data)
		m.setvol(int(v) * 100 / 255)
	# select a track from playlist
	elif cmd == 'SLCT':
		f = '>4sH'
		k, p = unpack(f, data)
		m.play(int(p))
	# toggle shuffle/random play
	elif cmd == 'SHFL':
		f = '>4sB'
		k, s = unpack(f, data)
		m.random(s)
	# toggle repeat play
	elif cmd == 'REPT':
		f = '>4sB'
		k, s = unpack(f, data)
		m.repeat(s)
	# clear the playlist
	elif cmd == 'RMAL':
		m.clear()
	# list a directory
	elif cmd == 'DLST':
		d = str(data[6:])
		d = d.replace('\\', '/')
		print 'DLST: ', repr(d)
		send_dir(d)
	# append a file, playlist or directory
	elif cmd == 'LADD':
		d = str(data[6:])
		d = d.replace('\\', '/')
		print 'LADD: ', repr(d)
		m.add(d)
	# play tracks from db
	elif cmd == 'PLAY':
		d = str(data[6:])
		d = d.replace('\\', '/')
		print 'PLAY: ', repr(d)
		s = m.status()
		id = s.playlist
		m.add(d)
		n = m.plchanges(int(id))
		m.playid(int(n[0].id))
	# shut me down
	elif cmd == 'SHUT':
		print 'Shut down request received.'
		break
	# unknown
	else:
	        print 'New Command [%s]' % repr(data)


client_sock.close()
server_sock.close()
print 'Shut down.'
